package com.jz.dm.service.impl;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.jz.common.constant.ResultMsg;
import com.jz.common.exception.ResponseException;
import com.jz.common.utils.HttpsUtils;
import com.jz.common.utils.RedisUtils;
import com.jz.common.utils.Result;
import com.jz.dm.common.constant.TagConstants;
import com.jz.dm.common.enums.GeneralStatusTypeEnum;
import com.jz.dm.common.enums.apiInterface.ApiInfoOutTypeEnum;
import com.jz.dm.common.enums.apiInterface.ApiStatusEnum;
import com.jz.dm.common.util.RandomUtil;
import com.jz.dm.mapper.*;
import com.jz.dm.models.domian.ApiInterface;
import com.jz.dm.models.domian.ApiInterfaceCustom;
import com.jz.dm.models.domian.ApiInterfaceFile;
import com.jz.dm.models.domian.ApiOpenApiEsFields;
import com.jz.dm.models.req.CreateFolderReq;
import com.jz.dm.models.req.make.ApiInterfaceReq;
import com.jz.dm.service.ApiInterfaceService;
import com.jz.dm.service.ProducerService;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.util.*;

import static com.jz.dm.common.enums.auth.AuthReqTypeEnum.fromAuthReqTypeEnum;

/**
 * @author ZC
 * @PACKAGE_NAME: com.jz.dm.service.impl
 * @PROJECT_NAME: jz-dm-parent
 * @NAME: ProducerServiceImpl
 * @DATE: 2020-12-23/16:57
 * @DAY_NAME_SHORT: 周三
 * @Description:
 **/
@Service("producerService")
@Slf4j
public class ProducerServiceImpl implements ProducerService {

    @Resource
    private ApiInterfaceMapper apiInterfaceMapper;
    @Resource
    private ApiSyncingDatasourceMapper apiSyncingDatasourceMapper;
    @Resource
    private ApiOpenApiEsFieldsMapper apiOpenApiEsFieldsMapper;
    @Resource
    private ApiOpenApiEsTagconfigMapper apiOpenApiEsTagconfigMapper;
    @Resource
    private ApiInterfaceCustomMapper apiInterfaceCustomMapper;
    @Resource
    private ApiInterfaceFileMapper apiInterfaceFileMapper;
    @Autowired
    private ApiInterfaceService apiInterfaceService;
    @Autowired
    private RedisUtils redisUtils;
    @Autowired
    private HttpsUtils httpsUtils;

    @Value("${dmp.openapi.timeout.max}")
    private int maxTimeout;
    @Value("${dmp.openapi.timeout.default}")
    private int defaultTimeout;


    /**
     * 数据表查询链接JSON
     */
    @Value("${data.select.jsonUrl}")
    private String jsonUrl;
    /**
     * 数据表查询链接flow流
     */
    @Value("${data.select.flowUrl}")
    private String flowUrl;

    /**
     * 获取文件列表
     *
     * @param projectId
     * @return
     */
    @Override
    public Result getFileCatalog(Long projectId) {
        return Result.of_success(apiInterfaceFileMapper.getFileFolderLevel(projectId));
    }


    /**
     * 创建文件夹
     *
     * @param req
     * @return
     */
    @Override
    public Result createProjectFolder(CreateFolderReq req) {
        if (null != req.getParentId()) {
            if (null == apiInterfaceFileMapper.selectById(req.getParentId())) {
                return Result.of_error("父级文件夹不存在!");
            }
        }
        if (StringUtils.isNotBlank(req.getFileSource()) &&
                "1".equals(req.getFileSource())) { //api制作创建文件夹
            if (null == req.getProjectId()) {
                return Result.of_error("项目id不存在!");
            }
            if (addFolder(req)){
                return Result.of_error(ResultMsg.INSERT_FAIL);
            }
        } else if (StringUtils.isNotBlank(req.getFileSource()) &&
                "2".equals(req.getFileSource())) { //组织创建文件夹
            if (addFolder(req)){
             return Result.of_error(ResultMsg.INSERT_FAIL);
            }
        }
        return Result.of_error(ResultMsg.INSERT_FAIL);
    }

    /**
     * 添加文件夹
     * @param req
     * @return
     */
    private boolean addFolder(CreateFolderReq req) {
        ApiInterfaceFile interfaceFile = new ApiInterfaceFile();
        BeanUtils.copyProperties(req, interfaceFile);
        interfaceFile.setStatus(GeneralStatusTypeEnum.VALID.name());
        if (apiInterfaceFileMapper.insert(interfaceFile) == 0) {
            return true;
        }
        return false;
    }

    /**
     * 获取APIID
     *
     * @return
     */
    @Override
    public Result getCustomApiId() {
        return Result.of_success(getApiKey());
    }


    /**
     * 数据查询/标签查询
     *
     * @param req
     * @return
     */
    @Override
    @Transactional(rollbackFor = Exception.class, propagation = Propagation.REQUIRES_NEW)
    public Result addBigDataMakeApi(ApiInterfaceReq req) {
        if ("10002".equals(req.getJoinType())) {//数据查询
            if (StringUtils.isBlank(req.getEsDataSource()) ||
                    StringUtils.isBlank(req.getEsDataBase()) ||
                    StringUtils.isBlank(req.getEsTable())) {
                return Result.of_error("数据源信息不存在!");
            }
        }
        if (null != req.getReqFrequency()) {
            if (req.getReqFrequency() <= 0) {
                return Result.of_error("限流次数不能小于或者等于零");
            }
        }
        if (StringUtils.isBlank(req.getSignType()) || "0".equals(req.getSignType())) {
            return Result.of_error("加密类型错误!");
        }
        if (StringUtils.isNotBlank(req.getReqType())) {
            if (null == fromAuthReqTypeEnum(req.getReqType())) {
                return Result.of_error("限流类型不存在!");
            }
        }
        //校验文件夹信息
        if (checkFileifExist(req.getFolderId(), req.getProjectId())) {
            return Result.of_error("目标文件夹不存在!");
        }
        //保存表字段信息
        saveTableField(req);
        if (null != req.getId()) {//更新
            if (saveUpdateAPiBaseData(req) || saveUpdateApiExtendData(req)) {
                throw ResponseException.of_error(ResultMsg.UPDATE_FAIL);
            }
        } else {  //保存
            if (saveApiExtendData(req) || saveApiBaseData(req)) {
                throw ResponseException.of_error(ResultMsg.INSERT_FAIL);
            }
        }
        //保存缓存信息
        saveObjOnRedis(req);
        return Result.of_success(ResultMsg.INSERT_SUCCESS);
    }

    /**
     * 数据表查询 && 保存表字段信息
     *
     * @param req
     */
    private void saveTableField(ApiInterfaceReq req) {
        QueryWrapper<ApiOpenApiEsFields> queryOpenApi = new QueryWrapper<>();
        queryOpenApi.eq("es_database", req.getEsDataBase());
        queryOpenApi.eq("es_table", req.getEsTable());
        queryOpenApi.eq("is_deleted", 0);
        List<ApiOpenApiEsFields> listOpenApiEsFields = apiOpenApiEsFieldsMapper.selectList(queryOpenApi);
        //如果存在先根据库，表删除，重新添加
        if (CollectionUtils.isNotEmpty(listOpenApiEsFields)) {
            Map paMap = new HashMap<>();
            paMap.put("es_database", req.getEsDataBase());
            paMap.put("es_table", req.getEsTable());
            apiOpenApiEsFieldsMapper.deleteByMap(paMap);
        }
        ArrayList<ApiOpenApiEsFields> listApiEsFields = new ArrayList<>();
        //保存字段表中的字段信息
        JSONArray json = JSONArray.parseArray(req.getTableFields());
        for (int i = 0; i < json.size(); i++) {
            JSONObject jsonObject = (JSONObject) json.get(i);
            ApiOpenApiEsFields fields = new ApiOpenApiEsFields();
            fields.setEsDatabase(req.getEsDataBase());
            fields.setEsTable(req.getEsTable());
            fields.setFieldName(jsonObject.get("name").toString());
            fields.setFieldType(jsonObject.get("type").toString());
            fields.setFieldDesc(jsonObject.get("desc").toString());
            fields.setCreateUser(req.getCreateUser());
            listApiEsFields.add(fields);
        }
        //保存字段表中的字段信息
        if (CollectionUtils.isNotEmpty(listApiEsFields)) {
            apiOpenApiEsFieldsMapper.insetBatch(listApiEsFields);
        }
    }

    /**
     * DMP保存api基本信息/自定义API
     *
     * @param req
     * @return
     */
    @Override
    @Transactional(rollbackFor = Exception.class, propagation = Propagation.REQUIRED)
    public Result saveInterfaceAPi(ApiInterfaceReq req) {
        if (StringUtils.isNotBlank(req.getReqType())) {
            if (null == fromAuthReqTypeEnum(req.getReqType())) {
                return Result.of_error("限流类型不存在!");
            }
        }
        if (StringUtils.isBlank(req.getSignType()) || "0".equals(req.getSignType())) {
            return Result.of_error("加密类型错误!");
        }
        if (null != req.getReqFrequency()) {
            if (req.getReqFrequency() <= 0) {
                return Result.of_error("限流次数不能小于或者等于零");
            }
        }
        //校验文件夹信息
        if (checkFileifExist(req.getFolderId(), req.getProjectId())) {
            return Result.of_error("目标文件夹不存在!");
        }
        if (saveApiBaseData(req)) {
            throw ResponseException.of_error("保存Api基本信息失败!");
        }
        if (saveApiExtendData(req)) {
            throw ResponseException.of_error("保存Api扩展信息失败!");
        }
        //缓存API信息
        saveObjOnRedis(req);
        return Result.of_success(ResultMsg.INSERT_SUCCESS);
    }

    /**
     * 保存APi扩展信息
     *
     * @param req
     * @return
     */
    private boolean saveApiExtendData(ApiInterfaceReq req) {
        ApiInterfaceCustom custom = new ApiInterfaceCustom();
        BeanUtils.copyProperties(req, custom);
        if (apiInterfaceCustomMapper.insert(custom) == 0) {
            return true;
        }
        return false;
    }

    /**
     * 保存ApiInterfance基本信息
     *
     * @param req
     * @return
     */
    private boolean saveApiBaseData(ApiInterfaceReq req) {
        ApiInterface apiInterface = new ApiInterface();
        BeanUtils.copyProperties(req, apiInterface);
        apiInterface.setStatus(ApiStatusEnum.DRAFT.name());//草稿状态
        apiInterface.setOutputType(ApiInfoOutTypeEnum.JSON.name());//输出类型默认为Json
        Long timeout = Long.parseLong(req.getTimeout());
        if (timeout == null || timeout == 0) {
            apiInterface.setTimeout(String.valueOf(defaultTimeout));
        } else if (timeout < 0) {
            throw ResponseException.of_error("接口超时时间最小0ms");
        } else if (timeout > maxTimeout) {
            throw ResponseException.of_error("接口超时时间最大5000ms");
        }
        apiInterface.setTimeout(req.getTimeout());
        if ("10002".equals(req.getJoinType())) {//数据表查询时添加
            if (ApiInfoOutTypeEnum.JSON.name().equals(req.getOutType())) {
                apiInterface.setTargetUrl(jsonUrl);
            } else if (ApiInfoOutTypeEnum.FLOW.name().equals(req.getOutType())) {
                apiInterface.setTargetUrl(flowUrl);
            }
        }
        if (apiInterfaceMapper.insert(apiInterface) == 0) {
            return true;
        }
        return false;
    }

    /**
     * 校验文件夹是否存在
     *
     * @param folderId
     * @param projectId
     */
    private Boolean checkFileifExist(Long folderId, Long projectId) {
        if (null == folderId) {
            return true;
        }
        QueryWrapper<ApiInterfaceFile> queryWra = new QueryWrapper<>();
        queryWra.eq("file_source", 1);//Api制作
        queryWra.eq("project_id", projectId);
        queryWra.eq("id", folderId);
        queryWra.eq("is_deleted", 0);
        ApiInterfaceFile apiInterfaceFile = apiInterfaceFileMapper.selectOne(queryWra);
        if (null == apiInterfaceFile) {
            return true;
        }
        return false;
    }

    /**
     * 保存修改的api基本信息
     *
     * @param req
     * @return
     */
    private Boolean saveUpdateAPiBaseData(ApiInterfaceReq req) {
        ApiInterface apiInterface = apiInterfaceMapper.selectById(req.getId());
        if (null == apiInterface) {
            throw ResponseException.of_error(ResultMsg.DATA_NOT_FOUND);
        }
        if (StringUtils.isNotBlank(req.getStatus())) {
            if (!canChange(apiInterface.getStatus(), req.getStatus())) {
                String errorMsg = String.format("api当前状态为%s不可变更为%s",
                        ApiStatusEnum.valueOf(apiInterface.getStatus()).getText(),
                        ApiStatusEnum.valueOf(req.getStatus()).getText());
                throw ResponseException.of_error(errorMsg);
            }
        }
        apiInterface.setApiType(req.getApiType());
        apiInterface.setReqFrequency(req.getReqFrequency());
        apiInterface.setVersion(RandomUtil.generateVersion(apiInterface.getVersion()));//版本自动累加
        apiInterface.setSignType(req.getSignType());
        apiInterface.setApiFunction(req.getApiFunction());
        apiInterface.setStatus(req.getStatus());
        apiInterface.setApiProtocl(req.getApiProtocl());//传输方式 https http
        apiInterface.setUpdateDate(new Date());
        apiInterface.setUpdateUser(req.getCreateUser());
        apiInterface.setTargetUrl(req.getTargetUrl());
        Long timeout = Long.parseLong(req.getTimeout());
        if (timeout == null || timeout == 0) {
            apiInterface.setTimeout(String.valueOf(defaultTimeout));
        } else if (timeout < 0) {
            throw ResponseException.of_error("接口超时时间最小0ms");
        } else if (timeout > maxTimeout) {
            throw ResponseException.of_error("接口超时时间最大5000ms");
        }
        apiInterface.setTimeout(req.getTimeout());
        if (apiInterfaceMapper.updateById(apiInterface) == 0) {
            return true;
        }
        return false;
    }

    /**
     * 保存更新数据扩展信息
     *
     * @param req
     * @return
     */
    private Boolean saveUpdateApiExtendData(ApiInterfaceReq req) {
        ApiInterface apiInterface = apiInterfaceMapper.selectById(req.getId());
        ApiInterfaceCustom interfaceCustom = null;
        if (null != apiInterface) {
            interfaceCustom = apiInterfaceCustomMapper.selectById(apiInterface.getApiCustomId());
        }
        if (null == interfaceCustom) {
            throw ResponseException.of_error("更新API扩展信息不存在!");
        }
        interfaceCustom.setMaxRow(req.getMaxRow());
        interfaceCustom.setPageRow(req.getPageRow());
        interfaceCustom.setUpdateUser(req.getUpdateUser());
        interfaceCustom.setRequestParam(req.getRequestParam());
        interfaceCustom.setResponseParam(req.getResponseParam());
        if (StringUtils.isNotBlank(req.getEsDataSource())) {
            interfaceCustom.setEsDataSource(req.getEsDataSource());
        }
        if (StringUtils.isNotBlank(req.getEsDataBase())) {
            interfaceCustom.setEsDataBase(req.getEsDataBase());
        }
        if (StringUtils.isNotBlank(req.getEsTable())) {
            interfaceCustom.setEsTable(req.getEsTable());
        }
        if (apiInterfaceCustomMapper.updateById(interfaceCustom) == 0) {
            return true;
        }
        return false;
    }

    /**
     * 判断状态是否改变
     *
     * @param currentStatus
     * @param toStatus
     * @return
     */
    private boolean canChange(String currentStatus, String toStatus) {
        if (StringUtils.equals(currentStatus, toStatus)) {
            log.info("api的状态一直，crrentStatus:{},toStatus:{}", currentStatus, toStatus);
            return true;
        }
        //已经失效
        if (StringUtils.equals(currentStatus, ApiStatusEnum.EXPIRY.name())) {
            log.info("api的状态是：{}，不能修改为：{}", currentStatus, toStatus);
            return false;
        }
        //toStatus 草稿  其它状态不能变更为草稿
        if (StringUtils.equals(toStatus, ApiStatusEnum.DRAFT.name())) {
            log.info("api的状态是：{}[{}]，不能修改为：{}[{}]", currentStatus,
                    ApiStatusEnum.DRAFT.getText(), toStatus, ApiStatusEnum.DRAFT.getText());
            return false;
        }
        return true;
    }


    /**
     * 获取不重复的apiKey
     *
     * @return
     */
    private String getApiKey() {
        String random = RandomUtil.getStringRandom(16);
        QueryWrapper<ApiInterface> query = new QueryWrapper<>();
        query.eq("api_key", random);
        query.eq("is_deleted", 0);
        ApiInterface apiInterface = apiInterfaceMapper.selectOne(query);
        if (null != apiInterface) {
            random = RandomUtil.getStringRandom(16);
        }
        return random;
    }

    /**
     * 缓存数据
     *
     * @param req
     */
    private void saveObjOnRedis(ApiInterfaceReq req) {
        if (StringUtils.isBlank(req.getApiKey()) ||
                StringUtils.isBlank(req.getOutType()) ||
                StringUtils.isBlank(req.getTargetUrl()) ||
                StringUtils.isBlank(req.getReqMethod())) {
            log.info("--------------缓存数据为空");
            return;
        }
        try {
            JSONObject object = new JSONObject();
            object.put("targetUrl", req.getTargetUrl());
            object.put("outputType", ApiInfoOutTypeEnum.JSON.name());
            object.put("reqMethod", req.getReqMethod());
            String apiKey = TagConstants.OPEN_API_CACHE_KEY + req.getApiKey();
            String paramKey = redisUtils.get(apiKey);
            if (null != paramKey) {
                redisUtils.del(apiKey);
            }
            redisUtils.set(apiKey, object.toString());
        } catch (Exception ex) {
            log.error("缓存api制作数据异常:{}", ex.getMessage());
            ex.printStackTrace();
        }
    }
    /* *//**
     * 数据银行制作API/数据包/自定义
     *
     * @param req
     * @return
     *//*
    @Override
    @Transactional(rollbackFor = Exception.class, propagation = Propagation.REQUIRES_NEW)
    public Result addDataBankApiInfo(MakeDataBankApiReq req) {
     *//*   QueryWrapper<ApiInterfaceFile> query = new QueryWrapper<>();
        query.eq("file_source", "2");
        query.eq("is_deleted", 0);
        List<ApiInterfaceFile> fileSource = apiInterfaceFileMapper.selectList(query);
        if (fileSource.size() == 0 || fileSource.size() >= 2) {
            return Result.of_error("文件夹信息异常!");
        }*//*
        if (StringUtils.isNotBlank(req.getOutputType())) {
            if (!req.getOutputType().contains(ApiInfoOutTypeEnum.JSON.name()) ||
                    !req.getOutputType().contains(ApiInfoOutTypeEnum.FLOW.name())) {
                return Result.of_error("输出类型不存在!");
            }
        }
        if (StringUtils.isNotBlank(req.getReqType())) {
            if (null == fromAuthReqTypeEnum(req.getReqType())) {
                return Result.of_error("限流类型不存在!");
            }
        }
        String signStr = dataBankToken + req.getJoinType();
        String signResult = Md5.encrypt(signStr, "");
        if (!signResult.equals(req.getSign())) {
            return Result.of_error("签名错误!");
        }
        String apiKey = getApiKey();
        switch (req.getJoinType()) {
            case "10006"://数据银行三方上传
                saveDataBanker(req, apiKey, "3");
                break;
            case "10007"://数据银行数据包上传
                if (StringUtils.isNotBlank(req.getFileLocation())) return Result.of_error("文件地址不存在!");
                saveDataBanker(req, apiKey, "2");
                break;
            case "10008"://数据银行制作大数据表
                saveDataBanker(req, apiKey, "1");
                break;
            default:
                return Result.of_error("接入类型不存在!");
        }
        return Result.of_success(ResultMsg.INSERT_SUCCESS, apiKey);
    }*/

    /*    *//**
     * 保存数据银行api信息并缓存
     *
     * @param req     请求参数
     * @param apiKey  apiKey
     * @param apiType api类型
     *//*
    private void saveDataBanker(MakeDataBankApiReq req, String apiKey, String apiType) {
        ApiInterface apiInterface = saveApiInterFace(req, apiKey, apiType);
        saveApiInterFaceCustom(req, apiKey, apiInterface.getId());
        JSONObject object = new JSONObject();
        object.put("targetUrl", req.getTargetUrl());
        object.put("outputType", req.getOutputType());
        object.put("reqMethod", req.getReqMethod());
        saveObjOnRedis(apiInterface.getApiKey(), object);
    }*/

    /*    *//**
     * 保存api基本信息
     *
     * @param req
     * @param apiKey
     * @param apiType
     * @return
     *//*
    private ApiInterface saveApiInterFace(MakeDataBankApiReq req,
                                          String apiKey, String apiType) {
        ApiInterface anInterface = new ApiInterface();
        anInterface.setApiType(apiType);//数据银行制作大数据表
        anInterface.setStatus(ProducerStatusTypeEnum.ISSUE.name());//发布
        anInterface.setApiKey(apiKey);//apiKey
        BeanUtils.copyProperties(req, anInterface);
        if (apiInterfaceMapper.insert(anInterface) == 0) {
            throw ResponseException.of(ResultCode.FAILURE, "保存api基本信息异常!");
        }
        return anInterface;
    }*/

    /*    *//**
     * 保存APi扩展信息
     *
     * @param req
     * @param apiKey
     * @param interFaceId
     * @return
     *//*
    private ApiInterfaceCustom saveApiInterFaceCustom(MakeDataBankApiReq req, String apiKey,
                                                      Long interFaceId) {
        ApiInterfaceCustom interfaceCustom = new ApiInterfaceCustom();
        interfaceCustom.setApiKey(apiKey);
        interfaceCustom.setApiInterfaceId(interFaceId);
        BeanUtils.copyProperties(req, interfaceCustom);
        if (apiInterfaceCustomMapper.insert(interfaceCustom) == 0) {
            throw ResponseException.of(ResultCode.FAILURE, "保存api扩展信息异常!");
        }
        return interfaceCustom;
    }*/

    /*    *//**
     * Api制作(实时接入)
     *
     * @param req
     * @return
     *//*
    @Override
    public Result addRealCustomApi(MakeRealCustomApiReq req) {

        return null;
    }*/
    /*   *//**
     * 校验文件夹信息
     *
     * @param projectId
     * @param parentId
     * @param userName
     * @return
     *//*
    private Long checkFileExist(Long projectId, Long parentId, String userName) {
        if (null == parentId) {
            throw ResponseException.of_error("父文件id不能为空");
        }
        if (null == projectId) {
            throw ResponseException.of_error("项目id不能为空");
        }
        QueryWrapper<ApiInterfaceFile> query = new QueryWrapper<>();
        query.eq("project_id", projectId);
        query.eq("parent_id", parentId);
        ApiInterfaceFile file = apiInterfaceFileMapper.selectOne(query);
        if (null == file && 0 != parentId) {
            throw ResponseException.of(ResultCode.FAILURE, "父类文件夹不存在!");
        }
        if (null == file) {
            //查询父类是否存在
            ApiInterfaceFile apiInterfaceFile =
                    apiInterfaceFileMapper.selectOne(new QueryWrapper<ApiInterfaceFile>().eq("parent_id", parentId));
            if (null == apiInterfaceFile && 0 == parentId) {
                file = new ApiInterfaceFile();
                file.setProjectId(projectId);
                file.setFileSource("1");//dmp
                file.setParentId(0L);
                file.setStatus(GeneralStatusTypeEnum.VALID.name());
                file.setCreateUser(userName);
                file.setRemark("创建文件夹");
                if (apiInterfaceFileMapper.insert(file) == 0) {
                    throw ResponseException.of_error("创建文件夹失败!");
                }
            } else {
                throw ResponseException.of(ResultCode.FAILURE, "父类文件信息不存在!");
            }
        }
        return file.getId();
    }
*/

    /* *//**
     * 获取数据源配置
     *
     * @param req
     * @return
     *//*
    @Override
    public Result getSourceConfig(SourceConfigReq req) {
        if (StringUtils.isBlank(req.getApiKey())) {
            return Result.of_success("Api不能为空");
        }
        if (StringUtils.isNotBlank(req.getReleaseId())) {
            ApiInterface anInterface = apiInterfaceMapper.selectOne(new QueryWrapper<ApiInterface>().eq("api_key", req.getApiKey()));
            if (null != anInterface) {
                return Result.of_error("存在重复的Apikey");
            }
        } else {
            QueryWrapper<ApiInterface> query = new QueryWrapper<>();
            query.eq("api_key", req.getApiKey());
            query.eq("is_deleted", 0);
            ApiInterface apiInterface = apiInterfaceMapper.selectOne(query);
            if (null != apiInterface) {
                return Result.of_error("存在重复的Apikey");
            }
        }
        return Result.of_success(ResultMsg.SUCCESS);
    }

    *//**
     * 获取数据源库配置
     *
     * @param id
     * @return
     *//*
    @Override
    public Result getSourceWarehouseList(Long id) {
        ApiSyncingDatasource datasource = apiSyncingDatasourceMapper.selectById(id);
        if (null == datasource) {
            return Result.of_error("数据源不存在!");
        }
        JSONArray result = new JSONArray();
        log.info("查询es标签类下的索引地址信息:" + datasource.getJdbcUrl() + "/es/get/indicess");
        String resp = httpsUtils.doGet(datasource.getJdbcUrl() + "/es/get/indicess", new HashMap<>());
        if (resp != null) {
            log.info("返回索引信息集合:" + resp);
            JSONArray json = JSONArray.parseArray(resp);
            for (int i = 0; i < json.size(); i++) {
                String database = json.get(i).toString();
                //除去已点开头的索引返回前端
                if (!database.startsWith(".")) {
                    result.add(database);
                }
            }
            log.info("过滤后索引信息集合:" + result.toJSONString());
        }

        return Result.of_success(result);
    }

    *//**
     * 获取某个索引下的type
     *
     * @param indices
     * @param id
     * @return
     *//*
    @Override
    public Result getIndicesTypes(String indices, Long id) {
        ApiSyncingDatasource datasource = apiSyncingDatasourceMapper.selectById(id);
        if (null == datasource) {
            return Result.of_error("数据源不存在!");
        }
        log.info("查询es标签类索引下的type的地址信息:" + datasource.getJdbcUrl() + "/es/get/indices/types");
        JSONObject param = new JSONObject();
        param.put("indices", indices);
        log.info("参数集合：" + param.toString());
        String resp = httpsUtils.submitPost(datasource.getJdbcUrl() + "/es/get/indices/types", param.toString());
        log.info("索引下type的信息集合:" + resp);
        return Result.of_success(resp);
    }

    *//**
     * 获取某个索引 type的fields
     *
     * @param indices
     * @param type
     * @param id
     * @return
     *//*
    @Override
    public Result getIndicesTypesFields(String indices, String type, Long id) {
        ApiSyncingDatasource datasource = apiSyncingDatasourceMapper.selectById(id);
        if (null == datasource) {
            return Result.of_error("数据源不存在!");
        }
        log.info("查询es标签类洗typeFields地址信息:" + datasource.getJdbcUrl() + "/es/get/type/fields");
        HashMap<String, String> herders = new HashMap<>();
        herders.put("Content-Encoding", "UTF-8");
        herders.put("Content-Type", "application/json");
        JSONObject param = new JSONObject();
        param.put("indices", indices);
        param.put("type", type);
        String resp = null;
        try {
            resp = httpsUtils.submitPost(datasource.getJdbcUrl() + "/es/get/type/fields", param, herders);
            log.info("索引下type的fields信息集合:" + resp);
        } catch (Exception e) {
            e.printStackTrace();
        }
        return Result.of_success(resp);
    }*/




 /*   @Override
    public Result saveObjOnRedis() {
        MakeDataBankApiReq bankApiReq = new MakeDataBankApiReq();
       *//* bankApiReq.setTransMode("HTTPS");
        bankApiReq.setSignType("MD5");
        bankApiReq.setJoinType("10004");
        bankApiReq.setOutputType("JSON");
        bankApiReq.setPage(false);
        bankApiReq.setReqType("DAY");
        bankApiReq.setTargetUrl("www.baidu.com");
        bankApiReq.setFileId(1L);
        bankApiReq.setSign("F2A8E3CFE528D6AE5C5B075046653F3E");
        bankApiReq.setReqFrequency(0L);
        bankApiReq.setApiDesc("4444");
        bankApiReq.setTimeout("5000");
        net.sf.json.JSONObject jsonObject = net.sf.json.JSONObject.fromObject(bankApiReq);
        redisUtils.set("apiKeyTest", jsonObject, 300);*//*
        JSONObject object = new JSONObject();
        object.put("targetUrl", "www.baidu.common");
        object.put("outputType", ApiInfoOutTypeEnum.JSON.name());
        saveObjOnRedis("apiKeyTest002", object);
        return Result.ok();
    }

    @Override
    public Result getObjOnRedis() {
        String apiKeyTest002 = redisUtils.get("apiKeyTest002");
        // JSONObject jsonObject = JSONObject.parseObject(apiKeyTest);
        JSONObject jsonObject = JSONObject.parseObject(apiKeyTest002);
        System.out.println(jsonObject);
        return Result.ok();
    }*/
}
